@@ -17,13 +17,18 @@ class APLocale(Locale):
       'item-battery_status': 'Battery',
       'item-battery_status_unknown': 'Unknown battery state',
       'item-battery_status_very_low': 'Battery level is very low',
-      'item-battery_status_full': 'Battery is full',
-      'item-battery_status_charging': 'Battery is charging',
-      'item-battery_status_charger_disconn': 'Charger disconnected',
+      'item-battery.charging_full': 'Battery is full',
+      'item-battery.charging_on': 'Battery is charging',
+      'item-battery.charging_off': 'Charger disconnected',
       'item-timeleft':  'Time left',
       'item-settings-translators': 'Translators',
-      'item-settings-about': 'About',
-      'unknown': 'unknown'
+      'settings/wrappers/battery': 'Battery',
+      'unknown': 'unknown',
+      'apmd-start': 'Start APMD',
+      'apmd-starting': 'Starting APMD',
+      'apmd-connected': 'APMD connected',
+      'apmd-disconnected': 'APMD disconnected',
+      'apmd-waiting': 'Waiting for APMD'
     }
 
 aplocale_singleton = APLocale()
